<?php

namespace Pinpoint\CarouselManager\Model\Resolver;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Exception\GraphQlNoSuchEntityException;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\Resolver\Value;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\Framework\Serialize\Serializer\Json as JsonSerializer;
use Pinpoint\CarouselManager\Model\Resolver\DataProvider\Carousel as CarouselDataProvider;
use Psr\Log\LoggerInterface;

/**
 * Define A GraphQL Schema for Carousel.
 */
class Carousel implements ResolverInterface
{
    /**
     * @var CarouselDataProvider
     */
    protected $carouselDataProvider;

    /**
     * @var JsonSerializer
     */
    protected $jsonSerializer;

    /**
     * @param CarouselDataProvider $carouselDataProvider
     */
    public function __construct(
        CarouselDataProvider $carouselDataProvider,
        JsonSerializer $jsonSerializer
    ) {
        $this->carouselDataProvider = $carouselDataProvider;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * Fetches the data from persistence models and format it according to the GraphQL schema.
     *
     * @param \Magento\Framework\GraphQl\Config\Element\Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return mixed|Value
     * @throws \Exception
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        if (!isset($args['carousel_id'])) {
            throw new GraphQlInputException(__('Carousel ID must be provided.'));
        }
        try {
            $carousel = $this->carouselDataProvider->getCarousel($args['carousel_id']);
            $slides = $this->carouselDataProvider->getSlides($args['carousel_id']);
            $stores = [];
            return [
                'name' => $carousel->getData('name'),
                'slug' => $carousel->getData('slug'),
                'stores' => $carousel->getData('store_ids') ?? [],
                'items' => $slides
            ];
        } catch (NoSuchEntityException $exception) {
            throw new GraphQlNoSuchEntityException(__($exception), $exception);
        }
    }
}
